#! /usr/bin/ruby -KN
#
#  yapo - Yet Another Ports Overview
#
#  Copyright (c) 2003-2006 Jrgen Daubert <juergen.daubert@t-online.de>
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
#  USA.
#


require 'ncurses'

def ncurses_init(color1=nil, color2=nil)
	Ncurses.initscr
	Ncurses.cbreak
	Ncurses.noecho
	Ncurses.nonl
	Ncurses.curs_set(1)
	if Ncurses.has_colors? then
		c1fg = Ncurses::COLOR_WHITE
		c1bg = Ncurses::COLOR_BLUE
		c2fg = Ncurses::COLOR_BLACK
		c2bg = Ncurses::COLOR_WHITE
		if color1 then
			c1 = color1.split
			begin
				c1fg = eval("Ncurses::COLOR_#{c1[0].upcase}")
				c1bg = eval("Ncurses::COLOR_#{c1[1].upcase}")
			rescue
			end
		end
		if color2 then
			c2 = color2.split
			begin
				c2fg = eval("Ncurses::COLOR_#{c2[0].upcase}")
				c2bg = eval("Ncurses::COLOR_#{c2[1].upcase}")
			rescue
			end
		end
		Ncurses.start_color
		Ncurses.init_pair(1, c1fg, c1bg)
		Ncurses.init_pair(2, c2fg, c2bg)
		Ncurses::use_default_colors
	end
end

def ncurses_finish
	Ncurses.echo
	Ncurses.nocbreak
	Ncurses.nl
	Ncurses.endwin
end


class Win

	attr_accessor :hidden 

	def initialize
		@callbacks = {}
		@hidden = true
		@@cols = Ncurses.COLS
		@@rows = Ncurses.LINES
		@w=Ncurses.newwin(1,1,1,1)
		Ncurses.leaveok(@w,false)
		Ncurses.keypad(@w,true)
	end

	def Win.sigwinch_handler
		exit if Ncurses.LINES<4 or Ncurses.COLS<4
		ObjectSpace.each_object(Win) do |win| 
			win.resize 
		end
	end

	def window_size_changed
		cols = Ncurses.COLS
		rows = Ncurses.LINES
		if @@cols==cols and @@rows==rows then
			return false
		else
			@@cols = cols
			@@rows = rows
			Win.sigwinch_handler
			Ncurses.doupdate
			return true
		end
	end

	def set_reverse(choice, cursor=0)
		if choice then 
			if Ncurses.has_colors? then
				Ncurses.wattron(@w, Ncurses.COLOR_PAIR(1+cursor))
				Ncurses.wbkgdset(@w, Ncurses.COLOR_PAIR(1+cursor))
			else
				Ncurses.wattron(@w,Ncurses::A_REVERSE)
				Ncurses.wbkgdset(@w,Ncurses::A_REVERSE)
			end
		else
			Ncurses.wattroff(@w,Ncurses::A_REVERSE)
			Ncurses.wbkgdset(@w,Ncurses::A_NORMAL)
		end
	end

	def cur_row ; Ncurses.getcury(@w)  ; end
	def refresh ; Ncurses.wrefresh(@w) ; end
	def getch   ; Ncurses.wgetch(@w)   ; end
	def clear   ; Ncurses.wclear(@w)   ; end

	def add_callback(*names, &action)
		names.each do |name|
			if @callbacks[name] then
				@callbacks[name].push(action)
			else
				@callbacks[name] = [action]
			end
		end
	end

	def check_callback(what)
		if @callbacks[what] then
			@callbacks[what].each { |cb| cb.call(what) }
		end
	end

	alias bind_key add_callback

end # Win


class SingleLine < Win

	def initialize(row)
		@str  = ["","","","",""]
		@pos  = []
		@@history = []
		@start_row  = row
		super()
		resize
		Ncurses.leaveok(@w,true)
	end

	def resize
		@cols = @@cols
		if @start_row < 0 then
			@row = @@rows+@start_row
		else
			@row = @start_row
		end
		Ncurses.wresize(@w,1,@cols)
		Ncurses.mvwin(@w,@row,0)
		update if not hidden
	end

	def set(pos,what)
		@str[pos]=what.to_s
		update
	end

	def clear
		@pos.clear
		@pos.push(0)
		@pos.push(@str[0].length+1)
		@pos.push((@cols-@str[2].length)/2)
		@pos.push(@cols-(@str[4]+@str[3]).length-1)
		@pos.push(@cols-@str[4].length)
		Ncurses.werase(@w)
	end

	def update
		clear
		@pos.each_with_index do |pos,index| 
			Ncurses.mvwprintw(@w,0,pos,@str[index]) if @str[index]!=""
		end
		Ncurses.wnoutrefresh(@w)
	end

	def print(what)
		Ncurses.werase(@w)
		Ncurses.mvwprintw(@w,0,0,what)
		Ncurses.wrefresh(@w)
	end

	def getstr(prompt)
		Ncurses.echo
		Ncurses.leaveok(@w,false)
		Ncurses.werase(@w)
		input = read_line(prompt,50)
		Ncurses.noecho
		Ncurses.leaveok(@w,true)
		return input
	end

	def read_line(prompt, max_len)
		y = 0 ; x = 0 ; pos = 0 ; hist_pos = -1 ; str = ""

		Ncurses.mvwaddstr(@w,0,0, prompt)
		x += prompt.length+1

		loop do
			Ncurses.wclrtoeol(@w)
			Ncurses.mvwaddstr(@w,y,x,str)
			Ncurses.wmove(@w,y,x+pos)
			ch = Ncurses.wgetch(@w)
			case ch
			when 27
				return nil
			when Ncurses::KEY_LEFT
				pos = [0, pos-1].max
			when Ncurses::KEY_RIGHT
				pos = [pos+1, str.length].min
			when Ncurses::KEY_ENTER, ?\n, ?\r, 13
				if str !~ /^ *$/ then
					@@history.unshift(str)
				end
				return str 
			when Ncurses::KEY_BACKSPACE, 127
				str = str[0...([0, pos-1].max)] + str[pos..-1]
				pos = [0, pos-1].max
				Ncurses.mvwaddstr(@w, y, x+str.length, " ")
			when Ncurses::KEY_DC
				if pos <= str.length and str != ""
					str = str[0...([0, pos].max)] + str[([str.length, pos+1].min)..-1]
					Ncurses.mvwaddstr(@w, y, x+str.length, " ")
				end
			when Ncurses::KEY_RESIZE
				window_size_changed
				Ncurses.wclear(@w)
				Ncurses.mvwaddstr(@w, 0,0, prompt)
			when Ncurses::KEY_UP, Ncurses::KEY_DOWN
				if @@history.length != 0 then
					ch == Ncurses::KEY_UP ? up = 1 : up = -1
					Ncurses.wmove(@w,y,x)
					Ncurses.wclrtoeol(@w)
					hist_pos = [[0, hist_pos+up].max, @@history.length-1].min
					str = @@history[hist_pos]
					pos = [0, str.length].max
				end
			when 32..255
				if (str.length < max_len)
					str[pos,0] = ch.chr
					pos += 1
				else
					Ncurses.mvwaddstr(@w, y, x+str.length, " ")
				end
			end 
		end
	end

end # SingleLine


class ListWindow < Win

	attr_accessor :col_widths

	def initialize(start_row,spare_lines)
		@col_widths = []
		@row = start_row
		@spare = spare_lines
		super()
		@hidden = true
		resize
	end

	def resize
		check_callback("resize")
		@height = @@rows-@row-@spare-1
		@cols  = @@cols
		Ncurses.wresize(@w,@height+1,@cols)
		Ncurses.mvwin(@w,@row,0)
		clear
		update if not @hidden
	end

	## What to display

	def set(list)
		@list  = list
		@max = @list.length-1
		@first_col = 0
		@list[0] ? @last_col=@list[0].length-1 : @last_col=0
		@offset = 0
		@rec   = 0
		clear
		redraw
		top
		check_callback("set")
	end

	## Create a string with format col_widths from a record

	def to_s(rec)
		str = ""
		if rec then
			rec[@first_col..@last_col].each_with_index do |field,index|
				f = field.gsub(/([^%])%([^%])/, '\1%%\2')
				l = f.length - field.length
				w = @col_widths[index+@first_col] 
				str += sprintf("%-#{w}.#{w}s %s", f, " "*l)
			end
		end
		return str
	end

	## Funktions for redraw, update etc.

	def redraw
		@list[@offset,@height+1].each_with_index do |rec,index|
			s = sprintf("%-*.*s", @cols,@cols, to_s(rec))
			Ncurses.mvwprintw(@w,index,0,s)
		end
		Ncurses.wnoutrefresh(@w)
	end

	def reverse(on)
		row = cur_row
		set_reverse(on, 1)
		s = sprintf("%-*.*s", @cols-1,@cols-1, to_s(@list[cur_rec]))
		Ncurses.mvwprintw(@w,row,0,s)
		set_reverse(false)
		Ncurses.wnoutrefresh(@w)
	end

	def update
		redraw
		move_to(@rec)
	end

	# Reread the data from disk
	# Stay on current record, if possible

	def reread
		save_rec  = cur_rec
		if @list[save_rec] then
			save_name = @list[save_rec][0]
		end
		save_off  = @offset
		if check_callback("reread") then
			@offset = save_off
			Ncurses.werase(@w)
			redraw
			if not find(save_name,true) then
				if save_rec > 0 then
					save_rec -= 1
				end
				move_to(save_rec)
			end
		end
	end

	# Find a string in column col of the list
	# and moves the cursor to this entry

	def find(str, exact_match=false, col=0)
		if str != "" then
			begin
				if not exact_match then
					find_str = /^#{str}.*/i
				else
					find_str = /^#{str}$/
				end
			rescue
				return nil
			else
				@list.each_with_index do |field,index| 
					if field[col] =~ find_str then
						move_to(index)
						return index
					end
				end
				return nil
			end
		end
	end

	## How to move in the list
	## The most complicated stuff at all ;)

	def move_to(row)
		redraw_needed = true
		if row < 0 then
			return if @offset == 0
			@offset = 0
			win_row = 0
		elsif row > @max
			return if cur_row+@offset == @max
			@offset = @max-@height
			if @offset < 0 then
				@offset = 0
				win_row = @max
			else
				win_row = row
			end
		elsif row > @offset+@height then
			if cur_row == @height then
				@offset = row-@height
				win_row = @height
			else
				if row < @max-@height
					@offset = row
					win_row = 0
				else
					@offset = @max-@height
					win_row = row-@offset
				end
			end
		elsif row < @offset then
			@offset = row
			win_row = 0
		else
			redraw_needed = false
			win_row = row-@offset
		end
		redraw if redraw_needed
		reverse(false)
		Ncurses.wmove(@w,win_row,0)
		@rec = cur_rec
		check_callback("move_to")
		reverse(true)
		Ncurses.doupdate
	end

	##  Cursor keys

	def page_down
		if cur_row < @height then
			bottom
		else
			move_to(@offset+2*@height)
		end
	end

	def page_up
		if cur_row > 0 then
			top
		else
			move_to(@offset-@height)
		end
	end

	def down    ; move_to(@offset+cur_row+1) ; end
	def up      ; move_to(@offset+cur_row-1) ; end
	def top     ; move_to(@offset)           ; end
	def bottom  ; move_to(@offset+@height)   ; end

	## Scroll the list horizontally

	def change_first_col(col)
		if @first_col + col < 0 then
			@first_col = 0
		elsif @first_col + col < @last_col then
			@first_col += col
			update
		end
	end

	## Get the current record number and the
	## total number of records

	def cur_rec ; cur_row+@offset ; end
	def no_recs ; @list.length    ; end


	## Main loop with predefined keys

	def main_loop
		while @w and ch = Ncurses.wgetch(@w)
			case ch
			when Ncurses::KEY_DOWN
				down
			when Ncurses::KEY_UP
				up
			when Ncurses::KEY_NPAGE
				page_down
			when Ncurses::KEY_PPAGE
				page_up
			when Ncurses::KEY_LEFT
				change_first_col(-1)
			when Ncurses::KEY_RIGHT
				change_first_col(+1)
			when Ncurses::KEY_RESIZE
				window_size_changed
			else
				check_callback(ch)
				break if @hidden
			end
		end
	end

end # ListWindow


class InfoWindow < ListWindow

	def append(text)
		if text and text.length != 0 then
			@list.concat(text)
			@list.push("#LINE#")
			@max = @list.length-1
		end
	end

	def draw_line
		@list.push("#LINE#")
		@max = @list.length-1
	end

	def redraw
		@list[@offset,@height+1].each_with_index do |rec,index|
			Ncurses.wmove(@w,index,0)
			if rec == "#LINE#" then
				Ncurses.whline(@w,0,@cols)
			else
				Ncurses.waddstr(@w,rec)
			end
		end
		Ncurses.wrefresh(@w)
	end

	def down
		if @offset+@height < @max then
			@offset += 1
			redraw
		end
	end

	def up
		if @offset > 0 then
			@offset -=1
			redraw
		end
	end

	def page_down
		if @offset+2*@height < @max then
			@offset += @height
			redraw
		elsif @offset+@height < @max then
			@offset = @max-@height
			redraw
		end
	end

	def page_up
		if @offset > @height then
			@offset -= @height
			redraw
		elsif @offset > 0 then
			@offset = 0
			redraw
		end
	end

	def move_to(row) ; Ncurses.wrefresh(@w) ; end

end # InfoWindow


class PrtGet

	attr_accessor :filter, :flag_filter, :deps, :sort_col

	def initialize(fields, cmd=nil)
		@filter = //
		@deps = []
		@sort_col = 1
		@fields = fields
		@select = "#{fields.join("|")}\n"

		if cmd
			prt_get = cmd.to_s
		else
			prt_get = "prt-get"
		end
		
		@prt_get_printf    = "#{prt_get} printf"
		@prt_get_path      = "#{prt_get} path"
		@prt_get_info      = "#{prt_get} info"
		@prt_get_deps      = "#{prt_get} quickdep"
		@prt_get_dependent = "#{prt_get} dependent --all"
	end

	def read
		@ports_all = []
		@ports = []
		@flags = {}
		%x(#{@prt_get_printf} '#{@select}').each_line do |p|
			port = p.chomp!.split("|")
			if port.length != @fields.length then 
				next 
			end
			if block_given? then
				yield port,@flags
			end
			@ports_all.push(port)
		end
		sort_ports
	end

	def sort_ports
		col = @sort_col-1
		@ports_all.sort! do |x,y|
			xc, yc = x[col], y[col]	
			compare = xc <=> yc
			if compare == 0
				x[0] <=> y[0]
			else
				if xc == "" or yc == ""
					-compare
				else
					compare
				end
			end
		end
		run_filter
		list_deps
	end

	def run_filter
		if @filter then
			if @flag_filter
				@ports = @ports_all.find_all {|x| @flags[x[0]].to_s =~ @filter}
			else
				@ports = @ports_all.find_all {|x| @filter.match(x.join(" "))}
			end
		else
			@ports = @ports_all.dup
		end
		return @ports
	end

	def filter_string
		if @filter.source != ""
			@flag_filter ? "[#{@filter.source}]" : "/#{@filter.source}/"
		else
			return ""
		end
	end

	def list_deps
		if @deps[0] then
			@ports = []
			@deps.each do |prt|
				@ports.push(@ports_all.find {|x| x[0] == prt})
			end
		end
		return @ports
	end

	def depends(port)
		@deps = %x(#{@prt_get_deps} #{port}).split(" ")
	end

	def dependent(port)
		@deps = []
		%x(#{@prt_get_dependent} #{port}).each_line do |prt|
			prt.chomp!
			@deps.push(prt) if prt != ""
			@deps.sort!
		end
		return @deps[0]
	end

	def name(rec_no)
		if @ports[rec_no] then
			return @ports[rec_no][0]
		end
	end

	def flag(rec_no)
		port_name = name(rec_no)
		if port_name then
			return @flags[port_name]
		end
	end

	def path(port)
		%x(#{@prt_get_path} #{port}).strip
	end

	def info(port)
		info = []
		%x(#{@prt_get_info} #{port}).each_line do |line|
			info.push("#{line}")
		end
		return info 
	end

	def cat(port, what)
		file = path(port) + "/" + what
		if FileTest.exist?(file) then
			doc = []
			File.new(file).each do |line|
				doc.push(line)
			end
			return doc
		end
	end

end # PrtGet


class Config

	attr_accessor :sys_cmds
	attr_reader   :fkeys

	def initialize(path)
		@cfg = {}
		@fkeys = []
		if File.exist?(path)
			@file = File.new(path)
		end
	end

	def cfg_function_key?(line)
		if line =~ /^\s*F[1-9][0-2]?\s/
			l = line.split
			if (l[1..4].all? {|i| i =~ /yes|no/}) and (l.length >= 6)
				@cfg[l[0]] = l[1..4].map {|i| i == "yes" ? true : false} 
				@cfg[l[0]] << l[5..-1].join(" ")
				@fkeys << l[0]
			end
		end
	end

	def cfg_sys_command?(line)
		if line =~ /^\s*#{sys_cmds.join("|")}\s/
			if l = line.split and l.length >= 2
				@cfg[l[0]] = [l[1..-1].join(" ")]
			end
		end
	end

	def parse
		if @file
			@file.each do |line|
				cfg_function_key?(line)
				cfg_sys_command?(line)
			end
		end
	end

	def get(key, port=nil, path=nil)
		res = []
		return false if not @cfg[key] 
		@cfg[key].each do |cmd|
			a = cmd
			if a.is_a?(String)
				a = a.dup
				a.gsub!('@PORT@', port) if port 
				a.gsub!('@PATH@', path) if path
			end
			res << a
		end
		return res
	end

end


## Function to run system commands 
def run_command(ask_before, wait_after, cmd, prompt=nil)
	doit = true
	Ncurses.def_prog_mode
	Ncurses.endwin
	$stdout.sync = true
	if ask_before then
		print "Run \"#{cmd}\" [y/n] ? "
		doit = false if gets !~ /[yY]/
	end
	if doit
		puts prompt if prompt
		system(cmd)
		if wait_after
			print "\nPress <ENTER> to continue .. "
			gets
		end
	end
	Ncurses.reset_prog_mode
	return doit
end


# Program Infos
prog_name = 'yapo'
prog_version = '0.3.2'

# Key help
key_help_list = 'q:Quit  <sp>:Info  g:Goto  fF:Filter  dD:Deps  a:All  iI:Install  m:reMove  s:Sort  r:Reread  ?:Help'
key_help_info = '<sp>:List'

# Which fields should we show
prt_get_fields = ["%n","%i","%d","%p","%M","%P","%R","%E","%O"]

# The colomn widths of the list
list_col_widths = [20,4,80,25,45,45]

# Block to create flags and modifiy @ports_all data
mod_ports_data = proc do |port,flags|
	case port[1]
		when "yes"  ; flag = "I"
		when "diff" ;	flag = "D"
		else 	flag = " "
	end
	case port[6]
		when "yes" ; flag += "R"
		else flag += " "
	end
	case port[7] + port[8]
		when /(yes)+/ ; flag += "P"
		else flag += " "
	end
	flags["#{port[0]}"] = flag
	(6..8).each {|c| port.delete_at(c)}
end



begin

	# Don't show warnings
	$VERBOSE=nil
	trap('INT') {puts; exit}

	# Read yapo config file
	cfg = Config.new('/usr/etc/yapo.conf')
	cfg.sys_cmds = ['PRTGET', 'INSTALL', 'UPDATE', 'DEPINST', 'REMOVE', 'COLOR1', 'COLOR2']
	cfg.parse

	# Configure prt-get
	prt_get = PrtGet.new(prt_get_fields, cfg.get("PRTGET")[0])

	# Init ncurses
	ncurses_init(cfg.get("COLOR1")[0], cfg.get("COLOR2")[0])


	# Top line, inverse
	w1 = SingleLine.new(0)
	w1.set_reverse(true)
	w1.set(0, key_help_list)
	w1.hidden = false

	# The main list window
	w2 = ListWindow.new(1,1)
	w2.hidden = false
	w2.col_widths = list_col_widths

	# Bottom line, inverse
	w3 = SingleLine.new(-1)
	w3.set_reverse(true)
	w3.set(0, "(#{prt_get.sort_col})")
	w3.print("Reading ports data ...")
	w3.hidden = false

	# Window for meta-data
	w4 = InfoWindow.new(1,1)
	w4.hidden = true

	# Callbacks
	w2.add_callback("move_to") {w3.set(4, "#{w2.cur_rec+1}/#{w2.no_recs}")}
	w2.add_callback("move_to") {w3.set(3, "[#{prt_get.flag(w2.cur_rec)}]")}
	w2.add_callback("move_to") {w3.set(2, prt_get.name(w2.cur_rec))}
	w2.add_callback("reread")  {w3.print("Reading ports data ...")}
	w2.add_callback("reread")  {w2.set(prt_get.read(&mod_ports_data))}
	w4.add_callback("set")     {w1.set(0, key_help_info)}

	# Now reads the data
	w2.set(prt_get.read(&mod_ports_data))
	
	# Show the program info until first key is pressed
	w3.print("* #{prog_name}-#{prog_version}")


	## Configure the key bindings

	# Bind the "q" key to exit
	w2.bind_key(113) { exit }
	
	# Bind the "r" key to reread data from disk
	w2.bind_key(114) { w2.reread }
	
	# Space toggles between list and info window
	w4.bind_key(32) {
		w4.hidden = true
		w2.hidden = false
	}

	w2.bind_key(32) { 
		port = prt_get.name(w2.cur_rec)
		if port then
			w2.hidden = true
			w4.hidden = false
			w4.set(prt_get.info(port))
			w4.draw_line
			w4.append(prt_get.cat(port, "README"))
			w4.append(prt_get.cat(port, "pre-install"))
			w4.append(prt_get.cat(port, "post-install"))
			w4.update
			w4.main_loop
			w1.set(0, key_help_list)
			w2.clear
			w2.update
		end
	}

	# Bind the "f" and "F" keys to setup a filter/flag filter
	w2.bind_key(102, 70) { |key|
		prt_get.deps = []
		w3.set_reverse(false)
		f = w3.getstr(key == 70 ? "Flag filter:" : "Filter:")
		w3.set_reverse(true)
		w3.update
		if f and f != prt_get.filter.source
			begin
				prt_get.filter = /#{f}/i
			rescue
				w3.print("Invalid regular expression: #{f}")
				w2.getch
				w2.update
			else
				prt_get.flag_filter = (key == 70 ? true : false)
				n = prt_get.name(w2.cur_rec)
				w3.set(1, prt_get.filter_string)
				w2.set(prt_get.run_filter)
				w2.find(n)
			end
		else
			w2.refresh
		end
	}

	# Bind the "g" and "/" key to the goto function
	w2.bind_key(103, 47) {
		w3.set_reverse(false)
		g = w3.getstr("Goto:")
		w3.set_reverse(true)
		w3.update
		if not g or not w2.find(g)
			w2.refresh
		end 
	}

	# Bind the "s" key to sort the list by column number
	w2.bind_key(115) {
		if prt_get.deps != [] then
			w3.print "Sorting not possible in this view"
			w2.getch
			w3.update
			w2.refresh
		else
			w3.set_reverse(false)
			sort = w3.getstr("Sort by column:")
			w3.set_reverse(true)
			w3.update
			if sort != "" then
				if sort =~ /^[1-#{list_col_widths.length}]?$/ then
					prt_get.sort_col = sort.to_i
					w3.set(0, "(#{prt_get.sort_col})")
					w2.set(prt_get.sort_ports)
				else
					w3.print("Invalid column: #{sort}")
					w2.getch
					w3.update
					w2.refresh
				end
			else
				w2.refresh
			end
		end
	}

	# Bind the "d" key to list dependencies for the current port
	w2.bind_key(100) {	
		n = prt_get.name(w2.cur_rec)
		w3.set(1,">#{n}<")
		prt_get.depends(n)
		w2.set(prt_get.list_deps)
		w2.find(n,true)
	}

	# Bind the "D" key to list all ports which depend on the 
	# current port
	w2.bind_key(68) {	
		n = prt_get.name(w2.cur_rec)
		if prt_get.dependent(n) then
			prt_get.list_deps
			w3.set(1,"<#{n}>")
			w2.set(prt_get.list_deps)
		end
	}

	# Bind the "a" key to show all ports, removing any filter
	w2.bind_key(97) {
		n = prt_get.name(w2.cur_rec)
		run_filter = false
		if prt_get.deps[0] != nil then
			prt_get.deps = []
			run_filter = true
		elsif  prt_get.filter != // then
			prt_get.filter = //
			run_filter = true
		end
		if run_filter then
			w2.set(prt_get.run_filter)
			w3.set(1, prt_get.filter_string)
			w2.refresh
			w2.find(n, true)
		end
	}

	# Bind the "i" key to install/update the current port
	# Configuration keys: INSTALL and UPDATE
	w2.bind_key(105) { 
		port = prt_get.name(w2.cur_rec) 
		if port then
			if prt_get.flag(w2.cur_rec) =~ /[ID]/ then
				cmd = cfg.get("UPDATE", port)
			else
				cmd = cfg.get("INSTALL", port)
			end
			if cmd
				if run_command(true, true, cmd[0])
					w2.reread
				else
					w2.update
				end
			end
		end
	}

	# Bind the "I" key to install the current port with deps
	# Configuration key: DEPINST
	w2.bind_key(73) { 
		port = prt_get.name(w2.cur_rec) 
		if port then
			if cmd = cfg.get("DEPINST", port)
				if run_command(true, true, cmd[0])
					w2.reread
				else
					w2.update
				end
			end
		end
	}

	# Bind the "m" key to remove the current port
	# Configuration key: REMOVE
	w2.bind_key(109) { 
		if prt_get.flag(w2.cur_rec) =~ /[ID]/ then
			port = prt_get.name(w2.cur_rec) 
			if port then
				if cmd = cfg.get("REMOVE", port)
					if run_command(true, true, cmd[0])
						w2.reread
					else
						w2.update
					end
				end
			end
		end
	}

	# Bind the ":" key to run a shell command
	w2.bind_key(58) {
		w3.set_reverse(false)
		cmd = w3.getstr("Run:")
		wait_after = true
		if cmd == "" then 
				cmd = "bash"
				prompt = "Type EXIT to return to yapo .."
				wait_after = false
		end
		w3.set_reverse(true)
		w3.update
		if cmd and cmd !~ /^ *$/ then
			cur_dir = Dir.getwd
			if port = prt_get.name(w2.cur_rec) then
				Dir.chdir(prt_get.path(port))
			end
			run_command(false, wait_after, cmd, prompt)
			Dir.chdir(cur_dir)
			w2.update
		else
			w2.refresh
		end
	}

	# Bind the "?" key to show the yapo man.page
	w2.bind_key(63) {
		run_command(false, false, "man yapo")
		w2.update
	}


	# Configuration of function keys F1-F12
	# cmd[0]: Ask before
	# cmd[1]: Wait after
	# cmd[2]: Change directory
	# cmd[3]: Reread ports tree
	# cmd[4]: Command to execute
	cfg.fkeys.each do |key|
		w2.bind_key(eval("Ncurses::KEY_#{key}")) {
			port = prt_get.name(w2.cur_rec)
			path = prt_get.path(port)
			cmd  = cfg.get(key, port, path)
			if cmd[2]
				cur_dir = Dir.getwd
				Dir.chdir(path)
			end
			run_command(cmd[0], cmd[1], cmd[4])
			Dir.chdir(cur_dir) if cur_dir
			if cmd[3]
				w2.reread
			else
				w2.update
			end
		}
	end


	## Here we are
	w2.main_loop

ensure

	ncurses_finish

end


# vim:ts=2:sw=2:noexpandtab
# End of file 
